job "garage-staging" {
  type = "system"
  #datacenters = [ "neptune", "pluton" ]
  datacenters = [ "neptune" ]

  priority = 80

  constraint {
    attribute = "${attr.cpu.arch}"
    value     = "amd64"
  }

  group "garage-staging" {
    network {
      port "s3" { static = 3990 }
      port "rpc" { static = 3991 }
      port "web" { static = 3992 }
    }

    update {
      max_parallel = 1
      min_healthy_time = "30s"
      healthy_deadline = "5m"
    }

    task "server" {
      driver = "docker"

      config {
        image = "dxflrs/amd64_garage:v0.6.0-rc1"
        command = "/garage"
        args = [ "server" ]
        network_mode = "host"
        volumes = [
          "/mnt/storage/garage-staging/data:/data",
          "/mnt/ssd/garage-staging/meta:/meta",
          "secrets/garage.toml:/etc/garage.toml",
        ]
        logging {
          type = "journald"
        }
      }

      template {
        data = file("../config/garage.toml")
        destination = "secrets/garage.toml"
      }

      resources {
        memory = 1000
        cpu = 1000
      }

      kill_signal = "SIGINT"
      kill_timeout = "20s"

      service {
        tags = [
          "garage-staging-api",
          "tricot garage-staging.home.adnab.me",
        ]
        port = 3990
        address_mode = "driver"
        name = "garage-staging-api"
        check {
          type = "tcp"
          port = 3990
          address_mode = "driver"
          interval = "60s"
          timeout = "5s"
          check_restart {
            limit = 3
            grace = "90s"
            ignore_warnings = false
          }
        }
      }

      service {
        tags = ["garage-staging-rpc"]
        port = 3991
        address_mode = "driver"
        name = "garage-staging-rpc"
        check {
          type = "tcp"
          port = 3991
          address_mode = "driver"
          interval = "60s"
          timeout = "5s"
          check_restart {
            limit = 3
            grace = "90s"
            ignore_warnings = false
          }
        }
      }

      service {
        tags = [
            "garage-staging-web",
            "tricot *.garage-staging-web.home.adnab.me",
            "tricot matrix.home.adnab.me/.well-known/matrix/server",
            "tricot rust-docs",
            "tricot-add-header Access-Control-Allow-Origin *",
        ]
        port = 3992
        address_mode = "driver"
        name = "garage-staging-web"
        check {
          type = "tcp"
          port = 3992
          address_mode = "driver"
          interval = "60s"
          timeout = "5s"
          check_restart {
            limit = 3
            grace = "90s"
            ignore_warnings = false
          }
        }
      }

      restart { 
        interval = "30m"  
        attempts = 10  
        delay    = "15s"  
        mode     = "delay"
      }
    }
  }
}
